/*  xfibresOptions.h

    Saad Jbabdi, Jennifer McNab , FMRIB Image Analysis Group

    Copyright (C) 1999-2010 University of Oxford  */

/*  Part of FSL - FMRIB's Software Library
    http://www.fmrib.ox.ac.uk/fsl
    fsl@fmrib.ox.ac.uk

    Developed at FMRIB (Oxford Centre for Functional Magnetic Resonance
    Imaging of the Brain), Department of Clinical Neurology, Oxford
    University, Oxford, UK


    LICENCE

    FMRIB Software Library, Release 6.0 (c) 2018, The University of
    Oxford (the "Software")

    The Software remains the property of the Oxford University Innovation
    ("the University").

    The Software is distributed "AS IS" under this Licence solely for
    non-commercial use in the hope that it will be useful, but in order
    that the University as a charitable foundation protects its assets for
    the benefit of its educational and research purposes, the University
    makes clear that no condition is made or to be implied, nor is any
    warranty given or to be implied, as to the accuracy of the Software,
    or that it will be suitable for any particular purpose or for use
    under any specific conditions. Furthermore, the University disclaims
    all responsibility for the use which is made of the Software. It
    further disclaims any liability for the outcomes arising from using
    the Software.

    The Licensee agrees to indemnify the University and hold the
    University harmless from and against any and all claims, damages and
    liabilities asserted by third parties (including claims for
    negligence) which arise directly or indirectly from the use of the
    Software or the sale of any products based on the Software.

    No part of the Software may be reproduced, modified, transmitted or
    transferred in any form or by any means, electronic or mechanical,
    without the express permission of the University. The permission of
    the University is not required if the said reproduction, modification,
    transmission or transference is done without financial return, the
    conditions of this Licence are imposed upon the receiver of the
    product, and all original and amended source code is included in any
    transmitted product. You may be held legally responsible for any
    copyright infringement that is caused or encouraged by your failure to
    abide by these terms and conditions.

    You are not permitted under this Licence to use this Software
    commercially. Use for which any financial return is received shall be
    defined as commercial use, and includes (1) integration of all or part
    of the source code or the Software into a product for sale or license
    by or on behalf of Licensee to third parties or (2) use of the
    Software or any derivative of it for research with the final aim of
    developing software products for sale or license to a third party or
    (3) use of the Software or any derivative of it for research with the
    final aim of developing non-software products for sale or license to a
    third party, or (4) use of the Software to provide any service to an
    external organisation for which payment is received. If you are
    interested in using the Software commercially, please contact Oxford
    University Innovation ("OUI"), the technology transfer company of the
    University, to negotiate a licence. Contact details are:
    fsl@innovation.ox.ac.uk quoting Reference Project 9564, FSL.*/

#if !defined(xfibresOptions_h)
#define xfibresOptions_h

#include <string>
#include <iostream>
#include <fstream>
#include <stdlib.h>
#include <stdio.h>
#include "utils/options.h"
#include "utils/log.h"
#include "utils/tracer_plus.h"

namespace Xfibres {

class xfibresOptions {
 public:
  static xfibresOptions& getInstance();
  ~xfibresOptions() { delete gopt; }

  Utilities::Option<bool> verbose;
  Utilities::Option<bool> help;
  Utilities::Option<std::string> logdir;
  Utilities::Option<bool> forcedir;
  Utilities::Option<std::string> datafile;
  Utilities::Option<std::string> maskfile;
  Utilities::Option<std::string> bvecsfile;
  Utilities::Option<std::string> bvalsfile;
  Utilities::Option<std::string> gradampfile;
  Utilities::Option<std::string> graddurfile;
  Utilities::Option<std::string> trfile;
  Utilities::Option<std::string> flipanglefile;
  Utilities::Option<std::string> t1file;
  Utilities::Option<std::string> t2file;
  Utilities::Option<std::string> b1file;
  Utilities::Option<bool>   fixt1t2;
  Utilities::Option<int> nfibres;
  Utilities::Option<float> fudge;
  Utilities::Option<int> njumps;
  Utilities::Option<int> nburn;
  Utilities::Option<int> nburn_noard;
  Utilities::Option<int> sampleevery;
  Utilities::Option<int> updateproposalevery;
  Utilities::Option<int> seed;
  Utilities::Option<bool> no_ard;
  Utilities::Option<bool> all_ard;
  Utilities::FmribOption<bool> f0;
  Utilities::FmribOption<bool> ardf0;

  void parse_command_line(int argc, char** argv,  Utilities::Log& logger);

 private:
  xfibresOptions();
  const xfibresOptions& operator=(xfibresOptions&);
  xfibresOptions(xfibresOptions&);

  Utilities::OptionParser options;

  static xfibresOptions* gopt;

};

 inline xfibresOptions& xfibresOptions::getInstance(){
   if(gopt == NULL)
     gopt = new xfibresOptions();

   return *gopt;
 }

 inline xfibresOptions::xfibresOptions() :
   verbose(std::string("-V,--verbose"), false,
	   std::string("switch on diagnostic messages"),
	   false, Utilities::no_argument),
   help(std::string("-h,--help"), false,
	std::string("display this message"),
	false, Utilities::no_argument),
   logdir(std::string("--ld,--logdir"), std::string("logdir"),
	  std::string("log directory (default is logdir)"),
	  false, Utilities::requires_argument),
   forcedir(std::string("--forcedir"),false,std::string("Use the actual directory name given - i.e. don't add + to make a new directory"),false,Utilities::no_argument),
   datafile(std::string("-k,--data,--datafile"), std::string("data"),
	    std::string("data file"),
	    true, Utilities::requires_argument),
   maskfile(std::string("-m,--mask, --maskfile"), std::string("nodif_brain_mask"),
	    std::string("mask file"),
	    true, Utilities::requires_argument),
   bvecsfile(std::string("-r,--bvecs"), std::string("bvecs"),
	     std::string("b vectors file"),
	     true, Utilities::requires_argument),
   bvalsfile(std::string("-b,--bvals"), std::string("bvals"),
	     std::string("b values file"),
	     true, Utilities::requires_argument),
   gradampfile(std::string("-G,--gradamp"),"",
	       std::string("Gradient amplitudes"),
	       true, Utilities::requires_argument),
   graddurfile(std::string("-d,--graddur"),"",
	       std::string("Gradient durations"),
	       true, Utilities::requires_argument),
   trfile(std::string("-T,--tr"),"",
	  std::string("TRs file"),
	  true, Utilities::requires_argument),
   flipanglefile(std::string("-a,--flipangle"),"",
	  std::string("Flip Angle File file"),
	  true, Utilities::requires_argument),
   t1file(std::string("--T1"),"",
	  std::string("T1 map"),
	  false, Utilities::requires_argument),
   t2file(std::string("--T2"),"",
	  std::string("T2 map"),
	  false, Utilities::requires_argument),
   b1file(std::string("--B1"),"",
	  std::string("B1 map"),
	  false, Utilities::requires_argument),
   fixt1t2(std::string("--fixt1t2"),false,
	   std::string("Fix t1 and t2"),
	   false, Utilities::no_argument),
   nfibres(std::string("--nf,--nfibres"),1,
	   std::string("Maximum number of fibres to fit in each voxel (default 1)"),
	   false,Utilities::requires_argument),
   fudge(std::string("--fudge"),1,
	 std::string("ARD fudge factor"),
	 false,Utilities::requires_argument),
   njumps(std::string("--nj,--njumps"),1250,
	  std::string("Num of jumps to be made by MCMC (default is 1250)"),
	  false,Utilities::requires_argument),
   nburn(std::string("--bi,--burnin"),1000,
	 std::string("Total num of jumps at start of MCMC to be discarded (default is 1000)"),
	 false,Utilities::requires_argument),
   nburn_noard(std::string("--bn,--burnin_noard"),0,
	       std::string("num of burnin jumps before the ard is imposed (default is 0)"),
	       false,Utilities::requires_argument),
   sampleevery(std::string("--se,--sampleevery"),25,
	       std::string("Num of jumps for each sample (MCMC) (default is 25)"),
	       false,Utilities::requires_argument),
   updateproposalevery(std::string("--upe,--updateproposalevery"),40,
		       std::string("Num of jumps for each update to the proposal density std (MCMC) (default is 40)"),
		       false,Utilities::requires_argument),
   seed(std::string("--seed"),8665904,std::string("seed for pseudo random number generator"),
	false,Utilities::requires_argument),
   no_ard(std::string("--noard"),false,std::string("Turn ARD off on all fibres"),
	  false,Utilities::no_argument),
   all_ard(std::string("--allard"),false,std::string("Turn ARD on on all fibres"),
	   false,Utilities::no_argument),
   f0(std::string("--f0"),false,std::string("Add to the model an unattenuated signal compartment"),false,Utilities::no_argument),
   ardf0(std::string("--ardf0"),false,std::string("Use ard on f0"),false,Utilities::no_argument),
   options(std::string("xfibres for DWSSFP"),std::string( "xfibres --help (for list of options)\n"))
     {
       try {
       options.add(verbose);
       options.add(help);
       options.add(logdir);
       options.add(forcedir);
       options.add(datafile);
       options.add(maskfile);
       options.add(bvecsfile);
       options.add(bvalsfile);
       options.add(gradampfile);
       options.add(graddurfile);
       options.add(trfile);
       options.add(flipanglefile);
       options.add(t1file);
       options.add(t2file);
       options.add(b1file);
       options.add(fixt1t2);
       options.add(nfibres);
       options.add(fudge);
       options.add(njumps);
       options.add(nburn);
       options.add(nburn_noard);
       options.add(sampleevery);
       options.add(updateproposalevery);
       options.add(seed);
       options.add(no_ard);
       options.add(all_ard);
       options.add(f0);
       options.add(ardf0);
     }
     catch(Utilities::X_OptionError& e) {
       options.usage();
       std::cerr << std::endl << e.what() << std::endl;
     }
     catch(std::exception &e) {
       std::cerr << e.what() << std::endl;
     }

   }
}

#endif
